@@ -4,11 +4,9 @@ import (
 	"fmt"
 	"io"
 	"io/ioutil"
-	"net"
 	"os"
 	"os/signal"
 	"runtime"
-	"strconv"
 	"strings"
 	"syscall"
 	"text/tabwriter"
@@ -17,11 +15,11 @@ import (
 	"github.com/sirupsen/logrus"
 	"github.com/urfave/cli"
 
-	ovncluster "github.com/openvswitch/ovn-kubernetes/go-controller/pkg/cluster"
-	"github.com/openvswitch/ovn-kubernetes/go-controller/pkg/config"
-	"github.com/openvswitch/ovn-kubernetes/go-controller/pkg/factory"
-	"github.com/openvswitch/ovn-kubernetes/go-controller/pkg/ovn"
-	util "github.com/openvswitch/ovn-kubernetes/go-controller/pkg/util"
+	ovncluster "github.com/ovn-org/ovn-kubernetes/go-controller/pkg/cluster"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/factory"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/ovn"
+	util "github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 
 	kexec "k8s.io/utils/exec"
 )
@@ -55,6 +53,7 @@ GLOBAL OPTIONS:{{range $title, $category := getFlagsByCategory}}
 func getFlagsByCategory() map[string][]cli.Flag {
 	m := map[string][]cli.Flag{}
 	m["Generic Options"] = config.CommonFlags
+	m["CNI Options"] = config.CNIFlags
 	m["K8s-related Options"] = config.K8sFlags
 	m["OVN Northbound DB Options"] = config.OvnNBFlags
 	m["OVN Southbound DB Options"] = config.OvnSBFlags
@@ -92,6 +91,7 @@ func main() {
 	c.Version = config.Version
 	c.CustomAppHelpTemplate = CustomAppHelpTemplate
 	c.Flags = config.CommonFlags
+	c.Flags = append(c.Flags, config.CNIFlags...)
 	c.Flags = append(c.Flags, config.K8sFlags...)
 	c.Flags = append(c.Flags, config.OvnNBFlags...)
 	c.Flags = append(c.Flags, config.OvnSBFlags...)
@@ -187,38 +187,25 @@ func runOvnKube(ctx *cli.Context) error {
 		panic(err.Error())
 	}
 
-	netController := ctx.Bool("net-controller")
 	master := ctx.String("init-master")
 	node := ctx.String("init-node")
-	nodePortEnable := ctx.Bool("nodeport")
 	clusterController := ovncluster.NewClusterController(clientset, factory)
 
-	if master != "" || node != "" {
-		clusterController.GatewayInit = ctx.Bool("init-gateways")
-		clusterController.GatewayIntf = ctx.String("gateway-interface")
-		clusterController.GatewayNextHop = ctx.String("gateway-nexthop")
-		clusterController.GatewaySpareIntf = ctx.Bool("gateway-spare-interface")
-		clusterController.LocalnetGateway = ctx.Bool("gateway-local")
-		clusterController.GatewayVLANID = ctx.Uint("gateway-vlanid")
-		clusterController.OvnHA = ctx.Bool("ha")
+	cleanupNode := ctx.String("cleanup-node")
+	if cleanupNode != "" {
+		if master != "" || node != "" {
+			panic("Cannot specify cleanup-node together with 'init-node or 'init-master'.")
+		}
 
-		clusterController.ClusterIPNet, err = parseClusterSubnetEntries(ctx.String("cluster-subnet"))
+		err = clusterController.CleanupClusterNode(cleanupNode)
 		if err != nil {
+			logrus.Errorf(err.Error())
 			panic(err.Error())
 		}
+		return nil
+	}
 
-		clusterServicesSubnet := ctx.String("service-cluster-ip-range")
-		if clusterServicesSubnet != "" {
-			var servicesSubnet *net.IPNet
-			_, servicesSubnet, err = net.ParseCIDR(
-				clusterServicesSubnet)
-			if err != nil {
-				panic(err.Error())
-			}
-			clusterController.ClusterServicesSubnet = servicesSubnet.String()
-		}
-		clusterController.NodePortEnable = nodePortEnable
-
+	if master != "" || node != "" {
 		if master != "" {
 			if runtime.GOOS == "windows" {
 				panic("Windows is not supported as master node")
@@ -229,6 +216,11 @@ func runOvnKube(ctx *cli.Context) error {
 				logrus.Errorf(err.Error())
 				panic(err.Error())
 			}
+			ovnController := ovn.NewOvnController(clientset, factory)
+			if err := ovnController.Run(); err != nil {
+				logrus.Errorf(err.Error())
+				panic(err.Error())
+			}
 		}
 
 		if node != "" {
@@ -242,83 +234,10 @@ func runOvnKube(ctx *cli.Context) error {
 				panic(err.Error())
 			}
 		}
-	}
-	if netController {
-		ovnController := ovn.NewOvnController(clientset, factory, nodePortEnable)
-		if clusterController.OvnHA {
-			err := clusterController.RebuildOVNDatabase(master, ovnController)
-			if err != nil {
-				logrus.Errorf(err.Error())
-				panic(err.Error())
-			}
-		}
-		if err := ovnController.Run(); err != nil {
-			logrus.Errorf(err.Error())
-			panic(err.Error())
-		}
-	}
-	if master != "" || netController {
-		// run forever
-		select {}
-	}
-	if node != "" {
+
 		// run forever
 		select {}
 	}
 
-	return nil
-}
-
-// parseClusterSubnetEntries returns the parsed set of CIDRNetworkEntries passed by the user on the command line
-// These entries define the clusters network space by specifying a set of CIDR and netmaskas the SDN can allocate
-// addresses from.
-func parseClusterSubnetEntries(clusterSubnetCmd string) ([]ovncluster.CIDRNetworkEntry, error) {
-	var parsedClusterList []ovncluster.CIDRNetworkEntry
-
-	clusterEntriesList := strings.Split(clusterSubnetCmd, ",")
-
-	for _, clusterEntry := range clusterEntriesList {
-		var parsedClusterEntry ovncluster.CIDRNetworkEntry
-
-		splitClusterEntry := strings.Split(clusterEntry, "/")
-		if len(splitClusterEntry) == 3 {
-			tmp, err := strconv.ParseUint(splitClusterEntry[2], 10, 32)
-			if err != nil {
-				return nil, err
-			}
-			parsedClusterEntry.HostSubnetLength = uint32(tmp)
-		} else if len(splitClusterEntry) == 2 {
-			// the old hardcoded value for backwards compatability
-			parsedClusterEntry.HostSubnetLength = 24
-		} else {
-			return nil, fmt.Errorf("cluster-cidr not formatted properly")
-		}
-
-		var err error
-		_, parsedClusterEntry.CIDR, err = net.ParseCIDR(fmt.Sprintf("%s/%s", splitClusterEntry[0], splitClusterEntry[1]))
-		if err != nil {
-			return nil, err
-		}
-
-		//check to make sure that no cidrs overlap
-		if cidrsOverlap(parsedClusterEntry.CIDR, parsedClusterList) {
-			return nil, fmt.Errorf("CIDR %s overlaps with another cluster network CIDR", parsedClusterEntry.CIDR.String())
-		}
-
-		parsedClusterList = append(parsedClusterList, parsedClusterEntry)
-
-	}
-
-	return parsedClusterList, nil
-}
-
-//cidrsOverlap returns a true if the cidr range overlaps any in the list of cidr ranges
-func cidrsOverlap(cidr *net.IPNet, cidrList []ovncluster.CIDRNetworkEntry) bool {
-
-	for _, clusterEntry := range cidrList {
-		if cidr.Contains(clusterEntry.CIDR.IP) || clusterEntry.CIDR.Contains(cidr.IP) {
-			return true
-		}
-	}
-	return false
+	return fmt.Errorf("need to run ovnkube in either master and/or node mode")
 }
