@@ -4,11 +4,16 @@ package cluster
 
 import (
 	"fmt"
-	"github.com/openvswitch/ovn-kubernetes/go-controller/pkg/factory"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/factory"
 )
 
 func initLocalnetGateway(nodeName string, clusterIPSubnet []string,
-	subnet string, nodePortEnable bool, wf *factory.WatchFactory) error {
+	subnet string, wf *factory.WatchFactory) error {
+	// TODO: Implement this
+	return fmt.Errorf("Not implemented yet on Windows")
+}
+
+func cleanupLocalnetGateway() error {
 	// TODO: Implement this
 	return fmt.Errorf("Not implemented yet on Windows")
 }
