@@ -5,7 +5,7 @@ import (
 	"fmt"
 	"strings"
 
-	util "github.com/openvswitch/ovn-kubernetes/go-controller/pkg/util"
+	util "github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 	"github.com/sirupsen/logrus"
 	kapi "k8s.io/api/core/v1"
 )
@@ -42,12 +42,9 @@ func (ovn *Controller) getDefaultGatewayLoadBalancer(protocol kapi.Protocol) str
 		return outStr
 	}
 
-	var gw string
-	gw, _, _ = util.RunOVNNbctl("--data=bare",
-		"--no-heading", "--columns=name", "find", "logical_router",
-		"options:lb_force_snat_ip=100.64.0.2")
-	if len(gw) == 0 {
-		logrus.Errorf("Error locating default gateway")
+	gw, _, err := util.GetDefaultGatewayRouterIP()
+	if err != nil {
+		logrus.Errorf(err.Error())
 		return ""
 	}
 
