@@ -2,7 +2,8 @@ package ovn
 
 import (
 	"fmt"
-	"github.com/openvswitch/ovn-kubernetes/go-controller/pkg/util"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 	"github.com/sirupsen/logrus"
 	kapi "k8s.io/api/core/v1"
 	"net"
@@ -34,13 +35,11 @@ func (ovn *Controller) syncServices(services []interface{}) {
 			continue
 		}
 
-		if service.Spec.Type != kapi.ServiceTypeClusterIP &&
-			service.Spec.Type != kapi.ServiceTypeNodePort &&
-			service.Spec.Type != kapi.ServiceTypeLoadBalancer {
+		if !util.ServiceTypeHasClusterIP(service) {
 			continue
 		}
 
-		if !util.IsServiceIPSet(service) {
+		if !util.IsClusterIPSet(service) {
 			logrus.Debugf("Skipping service %s due to clusterIP = %q",
 				service.Name, service.Spec.ClusterIP)
 			continue
@@ -52,7 +51,7 @@ func (ovn *Controller) syncServices(services []interface{}) {
 				protocol = TCP
 			}
 
-			if service.Spec.Type == kapi.ServiceTypeNodePort {
+			if util.ServiceTypeHasNodePort(service) {
 				port := fmt.Sprintf("%d", svcPort.NodePort)
 				if protocol == TCP {
 					nodeportServices[TCP] = append(nodeportServices[TCP], port)
@@ -167,7 +166,7 @@ func (ovn *Controller) syncServices(services []interface{}) {
 }
 
 func (ovn *Controller) deleteService(service *kapi.Service) {
-	if !util.IsServiceIPSet(service) || len(service.Spec.Ports) == 0 {
+	if !util.IsClusterIPSet(service) || len(service.Spec.Ports) == 0 {
 		return
 	}
 
@@ -175,7 +174,7 @@ func (ovn *Controller) deleteService(service *kapi.Service) {
 
 	for _, svcPort := range service.Spec.Ports {
 		var port int32
-		if service.Spec.Type == kapi.ServiceTypeNodePort {
+		if util.ServiceTypeHasNodePort(service) {
 			port = svcPort.NodePort
 		} else {
 			port = svcPort.Port
@@ -191,7 +190,7 @@ func (ovn *Controller) deleteService(service *kapi.Service) {
 
 		// targetPort can be anything, the deletion logic does not use it
 		var targetPort int32
-		if service.Spec.Type == kapi.ServiceTypeNodePort && ovn.nodePortEnable {
+		if util.ServiceTypeHasNodePort(service) && config.Gateway.NodeportEnable {
 			// Delete the 'NodePort' service from a load-balancer instantiated in gateways.
 			err := ovn.createGatewaysVIP(string(protocol), port, targetPort, ips)
 			if err != nil {
@@ -199,7 +198,7 @@ func (ovn *Controller) deleteService(service *kapi.Service) {
 					"%s:%d %+v", service.Name, port, err)
 			}
 		}
-		if service.Spec.Type == kapi.ServiceTypeNodePort || service.Spec.Type == kapi.ServiceTypeClusterIP {
+		if util.ServiceTypeHasClusterIP(service) {
 			loadBalancer, err := ovn.getLoadBalancer(protocol)
 			if err != nil {
 				logrus.Errorf("Failed to get load-balancer for %s (%v)",
